<?php

namespace Bydedi\Routing;

use Symfony\Component\HttpFoundation\Request;

/**
 * LocaleResolver class
 *
 * PHP Version 5.4
 *
 * @package  Bydedi\Routing
 * @author   Matthieu Bouquet <matthieu@dedi-agency.com>
 *
 */
class LocaleResolver
{
    /**
     * @var array
     */
    protected $languages;

    public function __construct(array $languages)
    {
        $this->languages = $languages;
    }

    public function resolve(Request $request)
    {
        $langs = [];
        $acceptLanguage = $request->server->get('HTTP_ACCEPT_LANGUAGE', null);
        $bestLocale = null;

        if (!is_null($acceptLanguage)) {
            // break up string into pieces (languages and q factors)
            preg_match_all('/([a-z]{1,8}(-[a-z]{1,8})?)\s*(;\s*q\s*=\s*(1|0\.[0-9]+))?/i', $acceptLanguage, $lang_parse);

            if (count($lang_parse[1])) {
                // create a list like "en" => 0.8
                $langs = array_combine($lang_parse[1], $lang_parse[4]);

                // set default to 1 for any without q factor
                foreach ($langs as $lang => $val) {
                    if ($val === '') $langs[$lang] = 1;
                }

                // sort list based on value
                arsort($langs, SORT_NUMERIC);
            }
        }
        // look through sorted list and use first one that matches our languages
        foreach ($langs as $lang => $val) {
            if (in_array($lang, array_keys($this->languages))) {
                $bestLocale = $lang;
                break;
            }
        }

        return $bestLocale;
    }
}