<?php

namespace Bydedi\Routing;

use Bydedi\Container\ServiceProvider;
use Symfony\Component\DependencyInjection\Reference;

class RoutingServiceProvider extends ServiceProvider
{

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {

        $this->serviceContainer->register('router.request_context', 'Symfony\Component\Routing\RequestContext');

        if (!empty($_SERVER['REQUEST_URI'])) {
            $this->serviceContainer->findDefinition('router.request_context')
                ->addArgument(urldecode($_SERVER['REQUEST_URI']));
        }

        $this->serviceContainer->register('router.file_locator', 'Symfony\Component\Config\FileLocator');

        $this->serviceContainer->register('router.cache', 'Doctrine\Common\Cache\ArrayCache');
        $this->serviceContainer->register('router.annotation_reader', 'Doctrine\Common\Annotations\AnnotationReader');

        $this->serviceContainer->register('router.cache_reader', 'Doctrine\Common\Annotations\CachedReader')
            ->addArgument(new Reference('router.annotation_reader'))
            ->addArgument(new Reference('router.cache'));

        $this->serviceContainer->register('router.annotation_class_loader', 'Bydedi\Routing\Loader\AnnotationClassLoader')
            ->addArgument(new Reference('router.cache_reader'));

        $this->serviceContainer->register('router.annotation_directory_loader', 'Symfony\Component\Routing\Loader\AnnotationDirectoryLoader')
            ->addArgument(new Reference('router.file_locator'))
            ->addArgument(new Reference('router.annotation_class_loader'));

        $paths = $this->serviceContainer->getParameter('app.paths');

        $this->serviceContainer->register('router', 'Bydedi\Routing\Router')
            ->setArguments(array(
                new Reference('router.annotation_directory_loader'),
                $paths,
                array(),
                new Reference('router.request_context')
            ));

        // data collector for debug bar
        $this->serviceContainer->register('data_collector.router', 'Bydedi\Routing\DataCollector\RouterDataCollector')
            ->addArgument(new Reference('router'))
            ->addTag('app.data_collector');

        // register template plugin
        $this->serviceContainer->register('template.plugins.route', 'Bydedi\Routing\TemplatePlugin\RouterPlugin')
            ->addArgument(new Reference('router'))
            ->addTag('app.template_plugin');
    }
}
